<?php

namespace Ems\Services;

use Doctrine\DBAL\Connection;
use Doctrine\ORM\EntityManager;
use Util\NovoCodigo;
use Util\Environment;
use Util\DateUtil;
use Ems\Util\{ ExpectedException, ArrayUtil, Utils};
use Ems\Repository\AdmissaoRepository;
use Ems\Services\OperadorService;
use Ems\Services\EstruturaService;
use Ems\Services\PessoaService;
use Ems\Services\DocumentoService;
use Ems\Services\ParceiroNegocioService;
use \Setforms2\Shared\ORM\DateTime;
use Ems\Util;
use Util\Util as UtilUtil;

class AdmissaoService {
    /** @var Environment */
    protected $environment;

    /** @var AdmissaoRepository */
    protected $admissaoRepository;

    /** @var OperadorService */
    protected $operadorService;

    /** @var EstruturaService */
    protected $estruturaService;

    /** @var PessoaService */
    protected $pessoaService;

    /** @var ParceiroNegocioService */
    protected $parceiroNegocioService;

    /** @var DocumentoService */
    protected $documentoService;

    /** @var Connection */
    protected $connection;

    public function __construct(
        Environment             $environment,
        AdmissaoRepository      $admissaoRepository,
        OperadorService         $operadorService,
        EstruturaService        $estruturaService,
        PessoaService           $pessoaService,
        ParceiroNegocioService  $parceiroNegocioService,
        EntityManager           $entityManager,
        DocumentoService        $documentoService
    ) {
        $this->environment              = $environment;
        $this->admissaoRepository       = $admissaoRepository;
        $this->operadorService          = $operadorService;
        $this->estruturaService         = $estruturaService;
        $this->pessoaService            = $pessoaService;
        $this->parceiroNegocioService   = $parceiroNegocioService;
        $this->connection               = $entityManager->getConnection();
        $this->documentoService         = $documentoService;
    }

    public function getVinculosNaoAprovados(
        string $cdOperador,
               $nrEstrutura
    ): array {
        $nrOrg = (int) $this->environment->getNrOrg();
        $nrOrgPadrao = (int) $this->environment->getNrOrgPadrao();

        $dtMesCompetencia = DateUtil::getPrimeiroDiaDoMes(new DateTime());
        $dtMesCompetencia = $dtMesCompetencia->format('d/m/Y');

        $isDP = $this->operadorService->operadorIsDP($cdOperador);

        $nrEstruturas = null;

        if (!$isDP) {
            $estruturas = $this->estruturaService->getEstruturasInferiores($nrEstrutura);
            $nrEstruturas = ArrayUtil::toStringSQL($estruturas);
        }

        $params = [
            'P_NRORG'           => $nrOrg,
            'P_NRORG_PADRAO'    => $nrOrgPadrao,
            'P_DTMESCOMPETENC'  => $dtMesCompetencia,
            'P_NRESTRUTURAS'    => $nrEstruturas,
        ];

        $admissoes = $this->admissaoRepository->getVinculosNaoAprovados(
            $params
        );

        return $admissoes;
    }

    public function setAdmissaoSimplificada(
        array $row
    ) {
        $dadosPessoais = (array) $row['dadosPessoais'];
        $enderecoRow = (array) $row['endereco'];
        $contaRow = (array) $row['conta'];
        $comuRow = (array) $row['comunicacao'];
        $documentos = (array) $row['documentos'];

        $cdOperador = (string) $row['CDOPERADOR'];

        $this->connection->beginTransaction();

        $dadosPessoais['NRCPFPESSOA'] = Utils::removePunctuation($dadosPessoais['NRCPFPESSOA']);

        try {
            $this->verificaPessoaCadastrada($dadosPessoais);
            $nrParcNegocio = $this->setPessoaParcNegocio($dadosPessoais, $cdOperador);
            $this->setPessoaAdmissao($dadosPessoais, $nrParcNegocio, $cdOperador);
            $this->setEnderecoAdmissao($enderecoRow, $nrParcNegocio, $cdOperador);
            $this->setContaCorrAdmissao($contaRow, $nrParcNegocio, $cdOperador);
            $this->setComunicaParcAdmissao($comuRow, $nrParcNegocio, $cdOperador);
            $this->setDocumentosAdmissao($documentos, $nrParcNegocio);

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollBack();
            throw $e;
        }

        return true;
    }

    /**
     * Verifica se já existe alguma pessoa cadastrada no sistema com o CPF ou PIS informado.
     * Caso positivo, será lançada uma exceção.
     */
    public function verificaPessoaCadastrada(
        array $dadosPessoais
    ) {
        $cpf = (string) $dadosPessoais['NRCPFPESSOA'];

        // Verificação necessária pois o campo não é obrigatório
        $pis = isset($dadosPessoais['NRPISPASEPPES']) ?
            (string) $dadosPessoais['NRPISPASEPPES'] :
            null;

        $this->validaCpfExistePessoa($cpf);

        if ($pis) {
            $this->validaPisExistePessoa($pis);
        }
    }

    /**
     * Verifica se já existe algum pessoa cadastrada com o CPF informado. Caso positivo, retorna true.
     */
    public function validaCpfExistePessoa(
        string $cpf,
        $throwException = true
    ): bool {
        $pessoa = $this->pessoaService->getPessoaByCpf($cpf);

        if ($pessoa) {
            if ($throwException) {
                $nome = (string) $pessoa['NMPESSOA'];
                $message = "Já existe uma pessoa cadastrada com o CPF $cpf! ($nome)";
                throw new ExpectedException($message);
            }

            return true;
        }

        return false;
    }

    /**
     * Verifica se já existe algum pessoa cadastrada com o PIS informado. Caso positivo, retorna true.
     */
    public function validaPisExistePessoa(
        string $pis,
        $throwException = true
    ): bool {
        $pessoa = $this->pessoaService->getPessoaByPis($pis);

        if ($pessoa) {
            if ($throwException) {
                $nome = (string) $pessoa['NMPESSOA'];
                $message = "Já existe uma pessoa cadastrada com o PIS $pis! ($nome)";
                throw new ExpectedException($message);
            }

            return true;
        }

        return false;
    }

    public function setPessoaParcNegocio(
        array $dadosPessoais,
        string $cdOperador
    ): int {
        $parcNegocioRow = [];

        $parcNegocioRow['NMPRINCIPALPARC'] = $dadosPessoais['NMPESSOA'];
        $parcNegocioRow['CDOPERINCLUSAO'] = $cdOperador;
        $parcNegocioRow['NMSECUNDARIPARC'] = $dadosPessoais['NMPESSOA'];
        $parcNegocioRow['DTNASCIFUNDPARC'] = $dadosPessoais['DTNASCPESSOA'];
        $parcNegocioRow['NRINSCRICAOPARC'] = $dadosPessoais['NRCPFPESSOA'];
        $parcNegocioRow['CDTIPOINSCRICAO'] = 'CPF';
        $parcNegocioRow['CDTIPOPARCPRINCIPAL'] = 'PESSOA';
        $parcNegocioRow['IDPESSOAFISICA'] = 'S';
        $parcNegocioRow['IDATIVO'] = 'S';
        $parcNegocioRow['IDINSTITUICAO'] = 'N';
        $parcNegocioRow['IDPARCFUNDIDO'] = 'N';

        $nrParcNegocio = $this->parceiroNegocioService->setParceiroNegocio($parcNegocioRow);

        return $nrParcNegocio;
    }

    /**
     * Cria registros nas tabela GPE_PESSOA e GPE_PESSOAH com os dados enviados.
     *
     * @param array $dadosPessoais Row com os dados pessoais da pessoa.
     * @param int $nrParcNegocio Número de parceiro de negócio do resgistro criado
     *  na tabela PARCNEGOCIO.
     *
     * @return int[] Array com os números dos registros criados nas tabelas GPE_PESSOA(0) e GPE_PESSOAH(1).
     */
    public function setPessoaAdmissao(
        array  $dadosPessoais,
        int    $nrParcNegocio,
        string $cdOperador
    ): array {
        $dadosPessoais['CDOPERINCLUSAO'] = $cdOperador;

        $nrPessoa = $this->setPessoaMasterAdmissao($dadosPessoais, $nrParcNegocio);
        $nrPessoaH = $this->setPessoaHistoryAdmissao($dadosPessoais, $nrPessoa);

        return [$nrPessoa, $nrPessoaH];
    }

    /**
     * Cria um registro na tabela GPE_PESSOA com os dados enviados.
     *
     * @param array $dadosPessoais Row com os dados pessoais da pessoa.
     * @param int $nrParcNegocio Número de parceiro de negócio do resgistro criado
     *  na tabela PARCNEGOCIO.
     *
     * @return int Número do registro criado na tabela GPE_PESSOA.
     */
    public function setPessoaMasterAdmissao(
        array  $dadosPessoais,
        int    $nrParcNegocio
    ): int {
        $rowPessoa = [];
        $rowPessoa['NRPARCNEGOCIO'] = $nrParcNegocio;

        $nrPessoa = $this->pessoaService->setPessoa($rowPessoa);

        return $nrPessoa;
    }

    /**
     * Cria um registro na tabela GPE_PESSOAH com os dados enviados.
     *
     * @param array $dadosPessoais Row com os dados pessoais da pessoa.
     * @param int $nrPessoa Número do registro criado na tabela GPE_PESSOA.
     *
     * @return int Número do registro criado na tabela GPE_PESSOAH.
     */
    public function setPessoaHistoryAdmissao(
        array $dadosPessoais,
        int $nrPessoa
    ): int {
        $dadosPessoais['NRPESSOA'] = $nrPessoa;
        $dadosPessoais['DTMESCOMPETENC'] = '01/01/3000';

        $nomeCompleto = $dadosPessoais['NMPESSOA'];
        $nomesPessoa = PessoaService::separarNomePessoa($nomeCompleto);

        $dadosPessoais['DSPRINOMEPESS']  = $nomesPessoa[0];
        $dadosPessoais['DSNOMEMEIOPESS'] = $nomesPessoa[1];
        $dadosPessoais['DSULTNOMEPESS']  = $nomesPessoa[2];

        $dadosPessoais['CDPAIS']  = $dadosPessoais['CDPAISNATURALI'];
        $dadosPessoais['SGESTADO']  = $dadosPessoais['SGESTADONATURALI'];
        $dadosPessoais['CDMUNICIPIO']  = $dadosPessoais['CDMUNICIPIONATURALI'];

        $nrPessoaH = $this->pessoaService->setPessoaH($dadosPessoais);

        return $nrPessoaH;
    }

    /**
     * Cria um registro na tabela ENDERECOPARC com os dados enviados.
     *
     * @param array $enderecoRow Row com os dados de endereço da pessoa.
     * @param int $nrParcNegocio Número de parceiro de negócio da pessoa.
     *
     * @return int Número do registro criado na tabela ENDERECOPARC.
     */
    public function setEnderecoAdmissao(
        array  $enderecoRow,
        int    $nrParcNegocio,
        string $cdOperador
    ) {
        $canSet =
            isset($enderecoRow['CDPAIS']);

        if ($canSet) {
            $enderecoRow['NRPARCNEGOCIO'] = $nrParcNegocio;
            $enderecoRow['CDOPERINCLUSAO'] = $cdOperador;
            $this->parceiroNegocioService->setEnderecoParc($enderecoRow);
        }
    }

    /**
     * Cria um registro na tabela CONTCORRPARC com os dados enviados.
     *
     * @param array $contaRow Row com os dados da conta corrente da pessoa.
     * @param int $nrParcNegocio Número de parceiro de negócio da pessoa.
     */
    public function setContaCorrAdmissao(
        array  $contaRow,
        int    $nrParcNegocio,
        string $cdOperador
    ) {
        $canSet =
            isset($contaRow['CDBANCO']) &&
            isset($contaRow['CDAGENCIA']) &&
            isset($contaRow['NRTPCONTCORR']) &&
            isset($contaRow['CDCONTCORR']);

        if ($canSet) {
            $contaRow['NRPARCNEGOCIO'] = $nrParcNegocio;
            $contaRow['CDOPERINCLUSAO'] = $cdOperador;
            $this->parceiroNegocioService->setContaCorrente($contaRow);
        }
    }

    /**
     * Cria na tabela COMUNICAPARC os registros de formas de comunicação da pessoa.
     *
     * @param array $comuRow Row com os dados das formas de comunicação da pessoa.
     * @param int $nrParcNegocio Número de parceiro de negócio da pessoa.
     */
    public function setComunicaParcAdmissao(
        array  $comuRow,
        int    $nrParcNegocio,
        string $cdOperador
    ) {
        // Telefone principal
        if (isset($comuRow['TELPRINCIPAL']) && $comuRow['TELPRINCIPAL'] !== '') {
            $telefonePrincipal = [
                'NRPARCNEGOCIO' => $nrParcNegocio,
                'CDOPERINCLUSAO' => $cdOperador,
                'CDFORMACOMU' => '01',
                'CDCOMUNICAPARC' => $comuRow['TELPRINCIPAL'],
                'CDPREFIXCOMUPARC' => $comuRow['PREFTELPRINCIPAL'] ?? null,
            ];

            $this->parceiroNegocioService->setComunicaParc($telefonePrincipal);
        }

        // Telefone residencial
        if (isset($comuRow['TELRESIDEN']) && $comuRow['TELRESIDEN'] !== '') {
            $telefoneResidencial = [
                'NRPARCNEGOCIO' => $nrParcNegocio,
                'CDOPERINCLUSAO' => $cdOperador,
                'CDFORMACOMU' => '03',
                'CDCOMUNICAPARC' => $comuRow['TELRESIDEN'],
                'CDPREFIXCOMUPARC' => $comuRow['PREFTELRESIDEN'] ?? null,
            ];

            $this->parceiroNegocioService->setComunicaParc($telefoneResidencial);
        }

        // Telefone comercial
        if (isset($comuRow['TELCOMERC']) && $comuRow['TELCOMERC'] !== '') {
            $telefoneComercial = [
                'NRPARCNEGOCIO' => $nrParcNegocio,
                'CDOPERINCLUSAO' => $cdOperador,
                'CDFORMACOMU' => '04',
                'CDCOMUNICAPARC' => $comuRow['TELCOMERC'],
                'CDPREFIXCOMUPARC' => $comuRow['PREFTELCOMERC'] ?? null,
            ];

            $this->parceiroNegocioService->setComunicaParc($telefoneComercial);
        }

        // Celular particular
        if (isset($comuRow['CELPARTIC']) && $comuRow['CELPARTIC'] !== '') {
            $celularParticular = [
                'NRPARCNEGOCIO' => $nrParcNegocio,
                'CDOPERINCLUSAO' => $cdOperador,
                'CDFORMACOMU' => '02',
                'CDCOMUNICAPARC' => $comuRow['CELPARTIC'],
                'CDPREFIXCOMUPARC' => $comuRow['PREFCELPARTIC'] ?? null,
            ];

            $this->parceiroNegocioService->setComunicaParc($celularParticular);
        }

        // Celular comercial
        if (isset($comuRow['CELCOMERC']) && $comuRow['CELCOMERC'] !== '') {
            $celularComercial = [
                'NRPARCNEGOCIO' => $nrParcNegocio,
                'CDOPERINCLUSAO' => $cdOperador,
                'CDFORMACOMU' => '06',
                'CDCOMUNICAPARC' => $comuRow['CELCOMERC'],
                'CDPREFIXCOMUPARC' => $comuRow['PREFCELCOMERC'] ?? null,
            ];

            $this->parceiroNegocioService->setComunicaParc($celularComercial);
        }

        // FAX
        if (isset($comuRow['FAX']) && $comuRow['FAX'] !== '') {
            $fax = [
                'NRPARCNEGOCIO' => $nrParcNegocio,
                'CDOPERINCLUSAO' => $cdOperador,
                'CDFORMACOMU' => '10',
                'CDCOMUNICAPARC' => $comuRow['FAX'],
                'CDPREFIXCOMUPARC' => $comuRow['PREFFAX'] ?? null,
            ];

            $this->parceiroNegocioService->setComunicaParc($fax);
        }

        // E-mail
        if (isset($comuRow['EMAIL']) && $comuRow['EMAIL'] !== '') {
            $email = [
                'NRPARCNEGOCIO' => $nrParcNegocio,
                'CDOPERINCLUSAO' => $cdOperador,
                'CDFORMACOMU' => '09',
                'CDCOMUNICAPARC' => $comuRow['EMAIL'],
            ];

            $this->parceiroNegocioService->setComunicaParc($email);
        }
    }

    /**
     * Salva os arquivos enviados do file server e guarda sua informações
     * na tabela GPE_ANEXOS.
     *
     * @param array $documentos Array com os documentos enviados pelo usuário.
     * @param int $nrParcNegocio Número de parceiro de negócio da pessoa.
     */
    public function setDocumentosAdmissao(
        array $documentos,
        int   $nrParcNegocio
    ) {
        foreach ($documentos as $documento) {
            $documento['NRPARCNEGOCIO'] = $nrParcNegocio;

            $this->documentoService->saveDocumento($documento);
        }
    }
}